unit FarbFilter;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StdCtrls, ExtCtrls, ComCtrls, Forms, Controls, LResources,
  Dialogs, ColorTools, Graphics, OPBitmap, LCLType, lazBridge;

type
 TBitmapArray = Array of TBitmap;
 PBitmapArray = ^TBitmapArray;
 TFarbFilter = class(TComponent)
  private
   Box          : TScrollBox;
   Headline     : TLabel;
   FilterColorL : TLabel;
   FilterColorE : TEdit;
   FilterColorB : TButton;
   NewColorL    : TLabel;
   NewColorE    : TEdit;
   NewColorB    : TButton;
   NewColorD    : TColorDialog;
   AbweichungL  : TLabel;
   AbweichungT  : TTrackBar;
   AbweichungE  : TEdit;
   RandL        : TLabel;
   RandE        : TEdit;
   GlanzColorL  : TLabel;
   GlanzColorT  : TTrackBar;
   GlanzColorE  : TEdit;
   GlanzRedL    : TLabel;
   GlanzRedE    : TEdit;
   Start        : TButton;
   StartSelf    : TButton;
   Progress     : TProgressBar;

   FilterColor : BGRAColor;
   NewColor    : BGRAColor;
   Abweichung  : byte;
   Rand        : word;
   GlanzColor  : Byte;
   GlanzRed    : word;

   Bitmaps   : PBitmapArray;
   Pos       : Integer;
   PosRead   : TMethod;
   PosWrite  : TMethod;
   GetColor  : TMethod;
   Operating : Boolean;
   Closed    : Boolean;
   procedure GlanzColorEChange(Sender : TObject);
   procedure GlanzColorTChange(Sender : TObject);
   procedure GlanzRedEChange(Sender : TObject);
   procedure RandEChange(Sender : TObject);
   procedure NewColorEChange(Sender : TObject);
   procedure FilterColorEChange(Sender : TObject);
   procedure AbweichungEChange(Sender : TObject);
   procedure AbweichungTChange(Sender : TObject);
   procedure NewColorBClick(Sender : TObject);
   procedure FilterColorBClick(Sender : TObject);
   procedure StartClick(Sender : TObject);
   procedure StartSelfClick(Sender : TObject);
   function DoFilter(Bild : TBitmap; Farbe,Filter : BGRAColor; _Abweichung : Byte; Schaerfe : byte; Red : byte; Width : Word) : TBitmap;
  public
   Vorschau : TImage;
   constructor Create(_Owner : TComponent; Choose,_Box : TScrollBox; ChooseAnchor : TImage; _Bitmaps : PBitmapArray; _PosRead,_PosWrite,_GetColor : Pointer);
   procedure Show(Sender : TObject);
   procedure InputColor(Color : BGRAColor);
   procedure Actualize;
   procedure Free;
 end;

implementation


type
 TPixCast = Array [0..1] of BGRAColor;
 PPixCast = ^TPixCast;
 TPosRead = function : integer of object;
 TPosWrite = procedure(Pos : integer) of object;
 TGetColor = procedure(Callback : TMethod) of object;

const
 Axis = 110;
 Steps = 8;
 _Steps = 4;

function EditToInt(Edit : TEdit) : integer;
var i : integer;
begin
 Edit.Color := $FFFFFF;
 Result := 0;
 if (Edit.Text = '') or (Edit.Text = '-') then exit;
 for i := 1 to Length(Edit.Text) do begin
  if (Edit.Text[i] <> '0') and
   (Edit.Text[i] <> '1') and
   (Edit.Text[i] <> '2') and
   (Edit.Text[i] <> '3') and
   (Edit.Text[i] <> '4') and
   (Edit.Text[i] <> '5') and
   (Edit.Text[i] <> '6') and
   (Edit.Text[i] <> '7') and
   (Edit.Text[i] <> '8') and
   (Edit.Text[i] <> '9') and
   (Edit.Text[i] <> '-') then begin Edit.Color := $A0A0FF; exit; end;
  if (Edit.Text[i] = '-') and (i <> 1) then begin Edit.Color := $A0A0FF; exit; end;
 end;
 Result := StrToInt(Edit.Caption);
end;

constructor TFarbFilter.Create(_Owner : TComponent; Choose,_Box : TScrollBox; ChooseAnchor : TImage; _Bitmaps : PBitmapArray; _PosRead,_PosWrite,_GetColor : Pointer);
begin
 inherited Create(_Owner);
 Closed := false;
 Bitmaps := _Bitmaps;
 Pos := -1;
 PosRead.Code := _PosRead;
 PosRead.Data := _Owner;
 PosWrite.Code := _PosWrite;
 PosWrite.Data := _Owner;
 GetColor.Code := _GetColor;
 GetColor.Data := _Owner;
 Operating := false;

 Box := _Box;
 Vorschau := TImage.Create(self);
 Vorschau.Parent := Choose;
 Vorschau.Top := 3;
 Vorschau.Left := 3;
 Vorschau.AnchorToNeighbour(akLeft,3,ChooseAnchor);
 Vorschau.Width := 80;
 Vorschau.Height := 80;
 Vorschau.Stretch := true;
 Vorschau.OnClick := @Show;
 Vorschau.Picture.LoadFromLazarusResource('FarbFilter');

 Cardinal(FilterColor) := $FF0000FF;
 Cardinal(NewColor) := $00000000;
 Abweichung := 50;
 Rand := 2;
 GlanzColor := 0;
 GlanzRed := 0;
end;

function TFarbFilter.DoFilter(Bild : TBitmap; Farbe,Filter : BGRAColor; _Abweichung : Byte; Schaerfe : byte; Red : byte; Width : Word) : TBitmap;
var
 Pic1,Pic2     : TCanvasOPBitmap;
 x,y           : integer;
 lines1,lines2 : Array of PPixCast;
 a,b,c,d,e,f   : smallint;
 i             : integer;
 s             : single;
 t             : single;
 Pix           : BGRAColor;
 RemBuf        : SmallInt;
 RemRes        : Word;
 Bit,Mask      : HBitmap;
 MAbweichung   : word;
 P1,P2,P3,P4   : word;

procedure _IsPixInFilter;
begin
 if ((Pix.R - Filter.R) * (Pix.R - Filter.R) < MAbweichung)
  and ((Pix.G - Filter.G) * (Pix.G - Filter.G) < MAbweichung)
  and ((Pix.B - Filter.B) * (Pix.B - Filter.B) < MAbweichung) then begin
   Lines1[y]^[x].A := 254;
   Pix := Farbe;
  end else if (Pix.A = 254) then begin
   Pix.R := Round(Farbe.R * (1 - Pix.A / 255) + Pix.R * Pix.A / 255);
   Pix.G := Round(Farbe.G * (1 - Pix.A / 255) + Pix.G * Pix.A / 255);
   Pix.B := Round(Farbe.B * (1 - Pix.A / 255) + Pix.B * Pix.A / 255);
   Pix.A := Round(Farbe.A * (1 - Pix.A / 255) + Pix.A * Pix.A / 255);
  end;
end;

procedure _RemoveColor;
begin
 RemRes := 0;
 if (s = 1) then exit;

 RemBuf := Round(Pix.R * (1 - s) + (Pix.R - Filter.R) * s);
 if (RemBuf < 0) then begin RemRes := RemRes + Pix.R; Pix.R := 0; end
 else begin RemRes := RemRes + Pix.R - RemBuf; Pix.R := RemBuf; end;

 RemBuf := Round(Pix.G * (1 - s) + (Pix.G - Filter.G) * s);
 if (RemBuf < 0) then begin RemRes := RemRes + Pix.G; Pix.G := 0; end
 else begin RemRes := RemRes + Pix.G - RemBuf; Pix.G := RemBuf; end;

 RemBuf := Round(Pix.B * (1 - s) + (Pix.B - Filter.B) * s);
 if (RemBuf < 0) then begin RemRes := RemRes + Pix.B; Pix.B := 0; end
 else begin RemRes := RemRes + Pix.B - RemBuf; Pix.B := RemBuf; end;

 if (RemRes > 255) then RemRes := 255;
end;

begin
 if (Operating) then exit;
 Operating := true;
 Result := TBitmap.Create;
 MAbweichung := _Abweichung * _Abweichung;

 // Bild laden
 if (Bild = nil) then exit;
 Pic1 := TCanvasOPBitmap.Create;
 Pic1.Height := Bild.Height;
 Pic1.Width := Bild.Width;
 AssignBitmapToOPBitmap(Bild,Pic1);
 Pic2 := TCanvasOPBitmap.Create;
 Pic2.Height := Pic1.Height;
 Pic2.Width := Pic1.Width;
 if (Pic1.PixelFormat <> pf32bit) then exit;
 SetLength(Lines1,Pic1.Height);
 SetLength(Lines2,Pic1.Height);

 // Farbe filtern
 Progress.Position := Round(5 / 100 * Progress.Width);
 for y := 0 to Pic1.Height - 1 do begin
  Lines1[y] := Pic1.ScanLine[y];
  Lines2[y] := Pic2.ScanLine[y];
  for x := 0 to Pic1.Width - 1 do begin
   Pix := Lines1[y]^[x];
   if (Pix.A = 254) then Lines1[y]^[x].A := 253;
   _IsPixInFilter;
   Lines2[y]^[x] := Pix;
  end;
  if (Pic1.Width > 2) then Progress.Position := Round(((y + 1) / (Pic1.Height) * 30 + 5) / 100 * Progress.Width);
  Application.ProcessMessages;
  if (not Operating) then begin Pic1.Free; Pic2.Free; exit; end;
 end;

 // Blauglanz entfernen
 Progress.Position := Round(36 / 100 * Progress.Width);
 if (Red > 0) and (Width > Steps) then begin
  y := _Steps;
  while (y < Pic1.Height) do begin
   x := _Steps;

   a := y - Width;
   while (a mod _Steps <> 0) do dec(a);
   if (a < 0) then a := 0;
   b := y + Width;
   while (b mod _Steps <> 0) do dec(b);
   if (b >= Pic1.Height) then b := Pic1.Height - 1;
   c := 0;
   d := x + Width - 1;
   while (d mod _Steps <> 0) do dec(d);
   if (d >= Pic1.Width) then d := Pic1.Width - 1;

   i := 0;
   e := a;
   while (e <= b) do begin
    f := c;
    while (f <= d) do begin
     f := f + _Steps;
     if (Lines1[e]^[f].a = 254) then inc(i);
    end;
    e := e + _Steps;
   end;

   while (x < Pic1.Width) do begin
    c := x - Width - 1;
    while ((c) mod _Steps <> 0) do dec(c);
    d := x + Width;
    while ((d) mod _Steps <> 0) do dec(d);
    if (d < Pic1.Width) then begin
     e := a;
     while (e <= b) do begin
      if (Lines1[e]^[d].a = 254) then inc(i);
      if (Lines1[e]^[d - _Steps].a = 254) then inc(i);
      e := e + _Steps;
     end;
    end;
    if (c >= _Steps) then begin
     e := a;
     while (e <= b) do begin
      if (Lines1[e]^[c].a = 254) then dec(i);
      if (Lines1[e]^[c - _Steps].a = 254) then dec(i);
      e := e + _Steps;
     end;
    end;

    if (i < 0) then i := 0;
    PWord(@Lines1[y]^[x])^ := i;
    if (x - Steps >= 0) and (y - Steps >= 0) then begin
     p1 := i;
     p2 := PWord(@Lines1[y-Steps]^[x])^;
     p3 := PWord(@Lines1[y-Steps]^[x-Steps])^;
     p4 := PWord(@Lines1[y]^[x-Steps])^;
     for c := 0 to Steps - 1 do for d := 0 to Steps - 1 do if (Lines1[y - c]^[x - d].A <> 254) then begin
      s := ((p1 * (1-c/Steps) + p2 * c/Steps) * (1-d/steps) + (p4 * (1-c/Steps) + p3 * c/Steps) * d/steps) / (Width * Width * 1020) * Red * _Steps * _Steps;
      Pix := Lines2[y - c]^[x - d];
      _RemoveColor;
      Lines1[y-c]^[x-d].R := RemRes;
      Lines2[y - c]^[x - d] := Pix;
     end;
    end;

    x := x + Steps;
   end;
   if (Pic1.Width > 2) then Progress.Position := Round(((y + 1) / (Pic1.Height) * 30 + 36) / 100 * Progress.Width);
   Application.ProcessMessages;
   if (not Operating) then begin Pic1.Free; Pic2.Free; exit; end;
   y := y + Steps;
  end;
 end;

 // Weichen Übergang
 Progress.Position := Round(67 / 100 * Progress.Width);
 if (Schaerfe > 0) then for y := 0 to Pic1.Height - 1 do begin
  x := 0;
  a := y - Schaerfe;
  if (a < 0) then a := 0;
  b := y + Schaerfe;
  if (b >= Pic1.Height) then b := Pic1.Height - 1;
  c := 0;
  d := x + Schaerfe - 1;
  if (d >= Pic1.Width) then d := Pic1.Width - 1;
  i := 0;
  for e := a to b do for f := c to d do if (Lines1[e]^[f].a = 254)
   then inc(i);
  for x := 0 to Pic1.Width - 1 do begin
   c := x - Schaerfe;
   d := x + Schaerfe;
   if (d < Pic1.Width) then for e := a to b do if (Lines1[e]^[d].a = 254) then inc(i);
   if (c > 0) then for e := a to b do if (Lines1[e]^[c-1].a = 254) then dec(i);

   pix := Lines2[y]^[x];
   if (Pix.A <> 254) then begin
    s := i / (Schaerfe * Schaerfe * 4) * 1.9;
    if (s > 1) then s := 1;
    if (s < 0) then s := 0;
    if (s > 0) then begin _RemoveColor; s := 1 - (1-s) * (1 - (RemRes + Lines1[y]^[x].R) / 255); end;
    Pix.R := Round(Pix.R * (1-s) + Farbe.R * s);
    Pix.G := Round(Pix.G * (1-s) + Farbe.G * s);
    Pix.B := Round(Pix.B * (1-s) + Farbe.B * s);
    Pix.A := Round(Pix.A * (1-s) + Farbe.A * s);
    Lines2[y]^[x] := Pix;
   end;
  end;
  if (Pic1.Width > 2) then Progress.Position := Round(((y + 1) / (Pic1.Height) * 30 + 67) / 100 * Progress.Width);
  Application.ProcessMessages;
  if (not Operating) then begin Pic1.Free; Pic2.Free; exit; end;
 end;

 // Bild in Bitmap schreiben
 Progress.Position := Round(98 / 100 * Progress.Width);
 Result.Height := Pic1.Height;
 Result.Width := Pic1.Width;
 AssignOPBitmapToBitmap(Pic2,Result);
 Pic1.Free;
 Progress.Position := Round(100 / 100 * Progress.Width);
 Operating := false;
end;

procedure TFarbFilter.Show(Sender : TObject);
begin
 Box.DestroyComponents;

 Headline := TLabel.Create(Box);
 Headline.Parent := Box;
 Headline.Font.Name := 'Arial Black';
 Headline.Font.Size := 20;
 Headline.Left := 3;
 Headline.Top := 3;
 Headline.Caption := 'Farbfilter';

 FilterColorE := TEdit.Create(Box);
 FilterColorE.Parent := Box;
 FilterColorE.Left := Axis;
 FilterColorE.AnchorToNeighbour(akTop,5,Headline);
 FilterColorE.Width := 100;
 FilterColorE.Text := BGRAToStr(Cardinal(FilterColor));
 FilterColorE.OnChange := @FilterColorEChange;

 FilterColorL := TLabel.Create(Box);
 FilterColorL.Parent := Box;
 FilterColorL.AnchorParallel(akTop,1,FilterColorE);
 FilterColorL.AnchorToNeighbour(akRight,3,FilterColorE);
 FilterColorL.Anchors := FilterColorL.Anchors - [akLeft];
 FilterColorL.Caption := 'Filter Farbe:';

 FilterColorB := TButton.Create(Box);
 FilterColorB.Parent := Box;
 FilterColorB.AnchorParallel(akTop,0,FilterColorE);
 FilterColorB.AnchorToNeighbour(akLeft,3,FilterColorE);
 FilterColorB.Width := 40;
 FilterColorB.Height := FilterColorE.Height;
 FilterColorB.Caption := '...';
 FilterColorB.OnClick := @FilterColorBClick;

 NewColorE := TEdit.Create(Box);
 NewColorE.Parent := Box;
 NewColorE.Left := Axis;
 NewColorE.AnchorToNeighbour(akTop,5,FilterColorE);
 NewColorE.Width := 100;
 NewColorE.Text := BGRAToStr(Cardinal(NewColor));
 NewColorE.OnChange := @NewColorEChange;

 NewColorL := TLabel.Create(Box);
 NewColorL.Parent := Box;
 NewColorL.AnchorParallel(akTop,1,NewColorE);
 NewColorL.AnchorToNeighbour(akRight,3,NewColorE);
 NewColorL.Anchors := NewColorL.Anchors - [akLeft];
 NewColorL.Caption := 'Neue Farbe:';

 NewColorB := TButton.Create(Box);
 NewColorB.Parent := Box;
 NewColorB.AnchorParallel(akTop,0,NewColorE);
 NewColorB.AnchorToNeighbour(akLeft,3,NewColorE);
 NewColorB.Width := 40;
 NewColorB.Height := NewColorE.Height;
 NewColorB.Caption := '...';
 NewColorB.OnClick := @NewColorBClick;

 NewColorD := TColorDialog.Create(Box);

 AbweichungT := TTrackBar.Create(Box);
 AbweichungT.Parent := Box;
 AbweichungT.Left := Axis;
 AbweichungT.AnchorToNeighbour(akTop,5,NewColorE);
 AbweichungT.TickMarks := tmBoth;
 AbweichungT.TickStyle := tsNone;
 AbweichungT.Min := 0;
 AbweichungT.Max := 255;
 AbweichungT.Width := 150;
 AbweichungT.Position := Abweichung;
 AbweichungT.OnChange := @AbweichungTChange;

 AbweichungL := TLabel.Create(Box);
 AbweichungL.Parent := Box;
 AbweichungL.AnchorToNeighbour(akRight,3,AbweichungT);
 AbweichungL.AnchorParallel(akTop,1,AbweichungT);
 AbweichungL.Anchors := AbweichungL.Anchors - [akLeft];
 AbweichungL.Caption := 'Abweichung:';

 AbweichungE := TEdit.Create(Box);
 AbweichungE.Parent := Box;
 AbweichungE.AnchorToNeighbour(akLeft,3,AbweichungT);
 AbweichungE.AnchorParallel(akTop,0,AbweichungT);
 AbweichungE.Width := 50;
 AbweichungE.Text := IntToStr(Abweichung);
 AbweichungE.OnChange := @AbweichungEChange;

 RandE := TEdit.Create(Box);
 RandE.Parent := Box;
 RandE.AnchorToNeighbour(akTop,5,AbweichungT);
 RandE.Left := Axis;
 RandE.Width := 100;
 RandE.Text := IntToStr(Rand);
 RandE.OnChange := @RandEChange;

 RandL := TLabel.Create(Box);
 RandL.Parent := Box;
 RandL.AnchorParallel(akTop,1,RandE);
 RandL.AnchorToNeighbour(akRight,3,RandE);
 RandL.Anchors := RandL.Anchors - [akLeft];
 RandL.Caption := 'Übergang:';

 GlanzColorT := TTrackBar.Create(Box);
 GlanzColorT.Parent := Box;
 GlanzColorT.Left := Axis;
 GlanzColorT.AnchorToNeighbour(akTop,5,RandE);
 GlanzColorT.TickMarks := tmBoth;
 GlanzColorT.TickStyle := tsNone;
 GlanzColorT.Min := 0;
 GlanzColorT.Max := 255;
 GlanzColorT.Width := 150;
 GlanzColorT.Position := GlanzColor;
 GlanzColorT.OnChange := @GlanzColorTChange;

 GlanzColorL := TLabel.Create(Box);
 GlanzColorL.Parent := Box;
 GlanzColorL.AnchorToNeighbour(akRight,3,GlanzColorT);
 GlanzColorL.AnchorParallel(akTop,1,GlanzColorT);
 GlanzColorL.Anchors := GlanzColorL.Anchors - [akLeft];
 GlanzColorL.Caption := 'Glanzreduktion:';

 GlanzColorE := TEdit.Create(Box);
 GlanzColorE.Parent := Box;
 GlanzColorE.AnchorToNeighbour(akLeft,3,GlanzColorT);
 GlanzColorE.AnchorParallel(akTop,0,GlanzColorT);
 GlanzColorE.Width := 50;
 GlanzColorE.Text := IntToStr(GlanzColor);
 GlanzColorE.OnChange := @GlanzColorEChange;

 GlanzRedE := TEdit.Create(Box);
 GlanzRedE.Parent := Box;
 GlanzRedE.AnchorToNeighbour(akTop,5,GlanzColorT);
 GlanzRedE.Left := Axis;
 GlanzRedE.Width := 100;
 GlanzRedE.Text := IntToStr(GlanzRed);
 GlanzRedE.OnChange := @GlanzRedEChange;

 GlanzRedL := TLabel.Create(Box);
 GlanzRedL.Parent := Box;
 GlanzRedL.AnchorParallel(akTop,1,GlanzRedE);
 GlanzRedL.AnchorToNeighbour(akRight,3,GlanzRedE);
 GlanzRedL.Anchors := GlanzRedL.Anchors - [akLeft];
 GlanzRedL.Caption := 'Reduktionsbreite:';

 Start := TButton.Create(Box);
 Start.Parent := Box;
 Start.Left := 20;
 Start.AnchorToNeighbour(akTop,7,GlanzRedE);
 Start.Width := 130;
 Start.Caption := 'START (Ausgangsbild)';
 Start.Enabled := false;
 Start.OnClick := @StartClick;

 StartSelf := TButton.Create(Box);
 StartSelf.Parent := Box;
 StartSelf.Left := 20;
 StartSelf.AnchorToNeighbour(akTop,3,Start);
 StartSelf.Width := 130;
 StartSelf.Caption := 'START (aktuelles Bild)';
 StartSelf.Enabled := false;
 StartSelf.OnClick := @StartSelfClick;

 Progress := TProgressBar.Create(Box);
 Progress.Parent := Box;
 Progress.AnchorParallel(akTop,0,Start);
 Progress.AnchorToNeighbour(akLeft,3,Start);
 Progress.Width := 150;
 Progress.Max := Progress.Width;

 Actualize;
end;

procedure TFarbFilter.Actualize;
var P : integer;
begin
 if (not Assigned(Start)) then exit;
 Start.Enabled := true;
 StartSelf.Enabled := false;
 P := TPosRead(PosRead)();
 if (P >= Length(Bitmaps^)) or (P < 0) then begin Start.Enabled := false; exit; end;
 if (not Assigned(Bitmaps^[P])) then Start.Enabled := false;
 Pos := P;
end;

procedure TFarbFilter.InputColor(Color : BGRAColor);
begin
 FilterColor := Color;
 if (not Assigned(FilterColorE)) then exit;
 FilterColorE.Text := BGRAToStr(Cardinal(Color));
end;

procedure TFarbFilter.NewColorEChange(Sender : TObject);
var i : integer;
begin
 NewColorE.Color := $FFFFFF;
 if (Length(NewColorE.Text) <> 8) then begin NewColorE.Color := $A0A0FF; exit; end;
 for i := 1 to Length(NewColorE.Text) do if (NewColorE.Text[i] <> '0') and
  (NewColorE.Text[i] <> '1') and
  (NewColorE.Text[i] <> '2') and
  (NewColorE.Text[i] <> '3') and
  (NewColorE.Text[i] <> '4') and
  (NewColorE.Text[i] <> '5') and
  (NewColorE.Text[i] <> '6') and
  (NewColorE.Text[i] <> '7') and
  (NewColorE.Text[i] <> '8') and
  (NewColorE.Text[i] <> '9') and
  (NewColorE.Text[i] <> 'a') and
  (NewColorE.Text[i] <> 'b') and
  (NewColorE.Text[i] <> 'c') and
  (NewColorE.Text[i] <> 'd') and
  (NewColorE.Text[i] <> 'e') and
  (NewColorE.Text[i] <> 'f') and
  (NewColorE.Text[i] <> 'A') and
  (NewColorE.Text[i] <> 'B') and
  (NewColorE.Text[i] <> 'C') and
  (NewColorE.Text[i] <> 'D') and
  (NewColorE.Text[i] <> 'E') and
  (NewColorE.Text[i] <> 'F') then begin NewColorE.Color := $A0A0FF; exit; end;
 Cardinal(NewColor) := StrToBGRA(NewColorE.Text);
end;

procedure TFarbFilter.FilterColorEChange(Sender : TObject);
var i : integer;
begin
 FilterColorE.Color := $FFFFFF;
 if (Length(FilterColorE.Text) <> 8) then begin FilterColorE.Color := $A0A0FF; exit; end;
 for i := 1 to Length(FilterColorE.Text) do if (FilterColorE.Text[i] <> '0') and
  (FilterColorE.Text[i] <> '1') and
  (FilterColorE.Text[i] <> '2') and
  (FilterColorE.Text[i] <> '3') and
  (FilterColorE.Text[i] <> '4') and
  (FilterColorE.Text[i] <> '5') and
  (FilterColorE.Text[i] <> '6') and
  (FilterColorE.Text[i] <> '7') and
  (FilterColorE.Text[i] <> '8') and
  (FilterColorE.Text[i] <> '9') and
  (FilterColorE.Text[i] <> 'a') and
  (FilterColorE.Text[i] <> 'b') and
  (FilterColorE.Text[i] <> 'c') and
  (FilterColorE.Text[i] <> 'd') and
  (FilterColorE.Text[i] <> 'e') and
  (FilterColorE.Text[i] <> 'f') and
  (FilterColorE.Text[i] <> 'A') and
  (FilterColorE.Text[i] <> 'B') and
  (FilterColorE.Text[i] <> 'C') and
  (FilterColorE.Text[i] <> 'D') and
  (FilterColorE.Text[i] <> 'E') and
  (FilterColorE.Text[i] <> 'F') then begin FilterColorE.Color := $A0A0FF; exit; end;
 Cardinal(FilterColor) := StrToBGRA(FilterColorE.Text);
end;

procedure TFarbFilter.AbweichungEChange(Sender : TObject);
var i : integer;
begin
 i := EditToInt(AbweichungE);
 if (i > 255) or (i < 0) then begin AbweichungE.Color := $A0A0FF; exit; end;
 if (i <> AbweichungT.Position) then AbweichungT.Position := i;
 Abweichung := i;
end;

procedure TFarbFilter.AbweichungTChange(Sender : TObject);
var i : integer;
begin
 i := EditToInt(AbweichungE);
 if (i <> AbweichungT.Position) then AbweichungE.Text := IntToStr(AbweichungT.Position);
 Abweichung := AbweichungT.Position;
end;

procedure TFarbFilter.RandEChange(Sender : TObject);
var i : integer;
begin
 i := EditToInt(RandE);
 if (i < 0) then begin RandE.Color := $A0A0FF; exit; end;
 Rand := i;
end;

procedure TFarbFilter.GlanzColorEChange(Sender : TObject);
var i : integer;
begin
 i := EditToInt(GlanzColorE);
 if (i > 255) or (i < 0) then begin GlanzColorE.Color := $A0A0FF; exit; end;
 if (i <> GlanzColorT.Position) then GlanzColorT.Position := i;
 GlanzColor := i;
end;

procedure TFarbFilter.GlanzColorTChange(Sender : TObject);
var i : integer;
begin
 i := EditToInt(GlanzColorE);
 if (i <> GlanzColorT.Position) then GlanzColorE.Text := IntToStr(GlanzColorT.Position);
 GlanzColor := GlanzColorT.Position;
end;

procedure TFarbFilter.GlanzRedEChange(Sender : TObject);
var i : integer;
begin
 i := EditToInt(GlanzRedE);
 if (i < 0) then begin GlanzRedE.Color := $A0A0FF; exit; end;
 GlanzRed := i;
end;

procedure TFarbFilter.FilterColorBClick(Sender : TObject);
var M : TMethod;
begin
 M.Code := @TFarbFilter.InputColor;
 M.Data := self;
 TGetColor(GetColor)(M);
end;

procedure TFarbFilter.NewColorBClick(Sender : TObject);
begin
 NewColorD.Execute;
 BGRAColor(NewColorD.Color).A := 255;
 NewColorE.Text := BGRAToStr(NewColorD.Color);
end;

procedure TFarbFilter.StartClick(Sender : TObject);
var i : integer;
begin
 for i := Pos + 1 to Length(Bitmaps^) - 1 do Bitmaps^[i].Free;
 SetLength(Bitmaps^,Pos + 2);
 Bitmaps^[Pos + 1] := DoFilter(Bitmaps^[Pos],NewColor,FilterColor,Abweichung,Rand,GlanzColor,GlanzRed);
 if (Closed) then exit;
 TPosWrite(PosWrite)(Pos + 1);
 StartSelf.Enabled := true;
end;

procedure TFarbFilter.StartSelfClick(Sender : TObject);
var i : integer;
begin
 if (Length(Bitmaps^) <= Pos + 1) then exit;
 for i := Pos + 2 to Length(Bitmaps^) - 1 do Bitmaps^[i].Free;
 SetLength(Bitmaps^,Pos + 2);
 Bitmaps^[Pos + 1] := DoFilter(Bitmaps^[Pos + 1],NewColor,FilterColor,Abweichung,Rand,GlanzColor,GlanzRed);
 if (Closed) then exit;
 TPosWrite(PosWrite)(Pos + 1);
end;

procedure TFarbFilter.Free;
begin
 Closed := true;
 Operating := false;
 inherited Free;
end;

initialization
{$I FarbFilter.lrs}

end.

