﻿
// Vorschaubild ist von http://www.wissen-digital.de/images/c/c1/Tier_634020.jpg

unit SchwarzAddition;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StdCtrls, ExtCtrls, ComCtrls, Forms, Controls, LResources,
  Dialogs, ColorTools, Graphics, OPBitmap, LCLType, lazBridge;

type
 TBitmapArray = Array of TBitmap;
 PBitmapArray = ^TBitmapArray;
 TSchwarzAddition = class(TComponent)
  private
   Box      : TScrollBox;
   Headline : TLabel;
   EffektL  : TLabel;
   EffektE  : TEdit;
   EffektT  : TTrackBar;
   Start    : TButton;
   Progress : TProgressBar;

   Effekt : Byte;

   Bitmaps   : PBitmapArray;
   Pos       : Integer;
   PosRead   : TMethod;
   PosWrite  : TMethod;
   Operating : Boolean;
   Closed    : Boolean;
   function DoFilter(Bild : TBitmap; _Effekt : Byte) : TBitmap;
   procedure EffektTChange(Sender : TObject);
   procedure EffektEChange(Sender : TObject);
   procedure StartClick(Sender : TObject);
  public
   Vorschau : TImage;
   constructor Create(_Owner : TComponent; Choose,_Box : TScrollBox; ChooseAnchor : TImage; _Bitmaps : PBitmapArray; _PosRead,_PosWrite : pointer);
   procedure Show(Sender : TObject);
   procedure InputColor(Color : BGRAColor);
   procedure Actualize;
   procedure Free;
 end;

implementation


type
 TPixCast = Array [0..1] of BGRAColor;
 PPixCast = ^TPixCast;
 TPosRead = function : integer of object;
 TPosWrite = procedure(Pos : integer) of object;

const
 Axis = 110;
 Steps = 8;
 _Steps = 4;

function EditToInt(Edit : TEdit) : integer;
var i : integer;
begin
 Edit.Color := $FFFFFF;
 Result := 0;
 if (Edit.Text = '') or (Edit.Text = '-') then exit;
 for i := 1 to Length(Edit.Text) do begin
  if (Edit.Text[i] <> '0') and
   (Edit.Text[i] <> '1') and
   (Edit.Text[i] <> '2') and
   (Edit.Text[i] <> '3') and
   (Edit.Text[i] <> '4') and
   (Edit.Text[i] <> '5') and
   (Edit.Text[i] <> '6') and
   (Edit.Text[i] <> '7') and
   (Edit.Text[i] <> '8') and
   (Edit.Text[i] <> '9') and
   (Edit.Text[i] <> '-') then begin Edit.Color := $A0A0FF; exit; end;
  if (Edit.Text[i] = '-') and (i <> 1) then begin Edit.Color := $A0A0FF; exit; end;
 end;
 Result := StrToInt(Edit.Caption);
end;

constructor TSchwarzAddition.Create(_Owner : TComponent; Choose,_Box : TScrollBox; ChooseAnchor : TImage; _Bitmaps : PBitmapArray; _PosRead,_PosWrite : pointer);
begin
 inherited Create(_Owner);
 Closed := false;
 Bitmaps := _Bitmaps;
 Pos := -1;
 PosRead.Code := _PosRead;
 PosRead.Data := _Owner;
 PosWrite.Code := _PosWrite;
 PosWrite.Data := _Owner;
 Operating := false;

 Box := _Box;
 Vorschau := TImage.Create(self);
 Vorschau.Parent := Choose;
 Vorschau.Top := 3;
 Vorschau.Left := 3;
 Vorschau.AnchorToNeighbour(akLeft,3,ChooseAnchor);
 Vorschau.Width := 80;
 Vorschau.Height := 80;
 Vorschau.Stretch := true;
 Vorschau.OnClick := @Show;
 Vorschau.Picture.LoadFromLazarusResource('SchwarzAddition');

 Effekt := 255;
end;

function TSchwarzAddition.DoFilter(Bild : TBitmap; _Effekt : Byte) : TBitmap;
var
 Pic1,Pic2     : TCanvasOPBitmap;
 x,y           : integer;
 lines1,lines2 : Array of PPixCast;
 s             : single;
 Pix           : BGRAColor;

procedure _Blend;
begin
 if ((Pix.R + Pix.G + Pix.B) / 1.7 < 255) then Pix.A := Round((Pix.R + Pix.G + Pix.B) / 1.7)
 else Pix.A := 255;
 if (Pix.A > 0) then begin
  if (Pix.R * 255 div Pix.A < 255) then Pix.R := Pix.R * 255 div Pix.A
  else Pix.R := 255;
  if (Pix.G * 255 div Pix.A < 255) then Pix.G := Pix.G * 255 div Pix.A
  else Pix.G := 255;
  if (Pix.B * 255 div Pix.A < 255) then Pix.B := Pix.B * 255 div Pix.A
  else Pix.B := 255;
 end;
 Pix.A := Pix.A * Pix.A div 255;
end;
begin
 if (Operating) then exit;
 Operating := true;
 Result := TBitmap.Create;

 // Bild laden
 if (Bild = nil) then exit;
 Pic1 := TCanvasOPBitmap.Create;
 Pic1.Height := Bild.Height;
 Pic1.Width := Bild.Width;
 AssignBitmapToOPBitmap(Bild,Pic1);
 Pic2 := TCanvasOPBitmap.Create;
 Pic2.Height := Pic1.Height;
 Pic2.Width := Pic1.Width;
 if (Pic1.PixelFormat <> pf32bit) then exit;
 SetLength(Lines1,Pic1.Height);
 SetLength(Lines2,Pic1.Height);

 // Transparenz-Farbaddition
 s := _Effekt / 255;
 Progress.Position := Round(10 / 100 * Progress.Width);
 for y := 0 to Pic1.Height - 1 do begin
  Lines1[y] := Pic1.ScanLine[y];
  Lines2[y] := Pic2.ScanLine[y];
  for x := 0 to Pic1.Width - 1 do begin
   Pix := Lines1[y]^[x];
   _Blend;
   Lines2[y]^[x].A := Round(Pix.A * s + Lines1[y]^[x].A * (1-s));
   Lines2[y]^[x].R := Round(Pix.R * s + Lines1[y]^[x].R * (1-s));
   Lines2[y]^[x].G := Round(Pix.G * s + Lines1[y]^[x].G * (1-s));
   Lines2[y]^[x].B := Round(Pix.B * s + Lines1[y]^[x].B * (1-s));
  end;
  if (Pic1.Width > 2) then Progress.Position := Round(((y + 1) / (Pic1.Height) * 85 + 10) / 100 * Progress.Width);
  Application.ProcessMessages;
  if (not Operating) then begin Pic1.Free; Pic2.Free; exit; end;
 end;

 // Bild in Bitmap schreiben
 Progress.Position := Round(95 / 100 * Progress.Width);
 Result.Height := Pic1.Height;
 Result.Width := Pic1.Width;
 AssignOPBitmapToBitmap(Pic2,Result);
 Pic1.Free;
 Progress.Position := Progress.Width;
 Operating := false;
end;

procedure TSchwarzAddition.Show(Sender : TObject);
begin
 Box.DestroyComponents;

 Headline := TLabel.Create(Box);
 Headline.Parent := Box;
 Headline.Font.Name := 'Arial Black';
 Headline.Font.Size := 20;
 Headline.Left := 3;
 Headline.Top := 3;
 Headline.Caption := 'Schwarz-Addition';

 EffektT := TTrackBar.Create(Box);
 EffektT.Parent := Box;
 EffektT.Left := Axis;
 EffektT.AnchorToNeighbour(akTop,5,Headline);
 EffektT.TickMarks := tmBoth;
 EffektT.TickStyle := tsNone;
 EffektT.Min := 0;
 EffektT.Max := 255;
 EffektT.Width := 150;
 EffektT.Position := Effekt;
 EffektT.OnChange := @EffektTChange;

 EffektE := TEdit.Create(Box);
 EffektE.Parent := Box;
 EffektE.AnchorToNeighbour(akLeft,3,EffektT);
 EffektE.AnchorParallel(akTop,0,EffektT);
 EffektE.Width := 100;
 EffektE.Text := IntToStr(Effekt);
 EffektE.OnChange := @EffektEChange;

 EffektL := TLabel.Create(Box);
 EffektL.Parent := Box;
 EffektL.AnchorParallel(akTop,1,EffektT);
 EffektL.AnchorToNeighbour(akRight,3,EffektT);
 EffektL.Anchors := EffektL.Anchors - [akLeft];
 EffektL.Caption := 'Effekt:';

 Start := TButton.Create(Box);
 Start.Parent := Box;
 Start.Left := 20;
 Start.AnchorToNeighbour(akTop,7,EffektT);
 Start.Width := 130;
 Start.Caption := 'START (Ausgangsbild)';
 Start.Enabled := false;
 Start.OnClick := @StartClick;

 Progress := TProgressBar.Create(Box);
 Progress.Parent := Box;
 Progress.AnchorParallel(akTop,0,Start);
 Progress.AnchorToNeighbour(akLeft,3,Start);
 Progress.Width := 150;
 Progress.Max := Progress.Width;

 Actualize;
end;

procedure TSchwarzAddition.Actualize;
var P : integer;
begin
 if (not Assigned(Start)) then exit;
 Start.Enabled := true;
 P := TPosRead(PosRead)();
 if (P >= Length(Bitmaps^)) or (P < 0) then begin Start.Enabled := false; exit; end;
 if (not Assigned(Bitmaps^[P])) then Start.Enabled := false;
 Pos := P;
end;

procedure TSchwarzAddition.InputColor(Color : BGRAColor);
begin
end;

procedure TSchwarzAddition.EffektEChange(Sender : TObject);
var i : integer;
begin
 i := EditToInt(EffektE);
 if (i > 255) or (i < 0) then begin EffektE.Color := $A0A0FF; exit; end;
 if (i <> EffektT.Position) then EffektT.Position := i;
 Effekt := i;
end;

procedure TSchwarzAddition.EffektTChange(Sender : TObject);
var i : integer;
begin
 i := EditToInt(EffektE);
 if (i <> EffektT.Position) then EffektE.Text := IntToStr(EffektT.Position);
 Effekt := EffektT.Position;
end;

procedure TSchwarzAddition.StartClick(Sender : TObject);
var i : integer;
begin
 for i := Pos + 1 to Length(Bitmaps^) - 1 do Bitmaps^[i].Free;
 SetLength(Bitmaps^,Pos + 2);
 Bitmaps^[Pos + 1] := DoFilter(Bitmaps^[Pos],Effekt);
 if (Closed) then exit;
 TPosWrite(PosWrite)(Pos + 1);
end;

procedure TSchwarzAddition.Free;
begin
 Closed := true;
 Operating := false;
 inherited Free;
end;

initialization
{$I SchwarzAddition.lrs}

end.

