

{ Die Buttons für Speichern / Öffnen entstammen Screenshots von Lazarus. }


unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, PairSplitter, Buttons, ComCtrls, StdCtrls, ColorTools, FarbFilter,
  SchwarzAddition, Punktentferner;

type

  { TForm1 }

  TForm1 = class(TForm)
    ButtonBack: TSpeedButton;
    ButtonForward: TSpeedButton;
    ButtonSave: TSpeedButton;
    PairSplitterSide: TCheckBox;
    DialogOpen: TOpenDialog;
    Bild: TImage;
    PairSplitter1: TPairSplitter;
    PairSplitterSide1: TPairSplitterSide;
    PairSplitterSide2: TPairSplitterSide;
    ButtonOpen: TSpeedButton;
    DialogSave: TSaveDialog;
    ToolWahl: TScrollBox;

    STool: TScrollBox;
    TrackBar1: TTrackBar;
    procedure Back(Sender: TObject);
    procedure BildMausDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BildMausMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure BildMausUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BildMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure BildMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure ButtonOpenClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DoForward(Sender: TObject);
    procedure ImgResize(Sender: TObject);
    procedure PToolFarbFilter(Sender: TObject);
    procedure Save(Sender: TObject);
    procedure SetZoom(Sender: TObject);
  private
   procedure OpenPic(const FileName : AnsiString);
   procedure DoPaint(Sender : TObject);
   procedure HideAll;
  public
   procedure WriteStepPos(Pos : integer);
   function ReadStepPos : integer;
   procedure GetColor(_Callback : TMethod);
  end; 

var
 Form1     : TForm1;
 Operating : Boolean = false;
 Picture   : TPicture;
 Steps     : Array of TBitmap;
 _StepPos  : integer = -1;
 Pos       : TRect;
 Zoom      : Single = 1;
 NewZoom   : Single;
 CurPos    : TPoint;
 FilterColor : BGRAColor;
 StartPicPos : TPoint;
 StartCurPos : TPoint;
 Moveing     : Boolean = false;
 Callback    : TMethod;
 Tool1 : TFarbFilter;
 Tool2 : TSchwarzAddition;
 Tool3 : TPunktentferner;



implementation

type
 TCallback = procedure(Color : BGRAColor) of object;

const
 Speed   = 1.1;
 MaxZoom = 10;
 MinZoom = 1;

procedure ActualizeTools;
begin
 Tool1.Actualize;
 Tool2.Actualize;
 Tool3.Actualize;
 Form1.ButtonSave.Enabled := true;
 if (_StepPos >= Length(Steps)) or (_StepPos < 0) then Form1.ButtonSave.Enabled := false;
end;

{ TForm1 }

procedure TForm1.HideAll;
begin
 STool.Visible := false;
end;

procedure TForm1.PToolFarbFilter(Sender: TObject);
begin
 HideAll;
 STool.Visible := true;
end;

procedure TForm1.Save(Sender: TObject);
begin
 if (_StepPos >= Length(Steps)) or (_StepPos < 0) then exit;
 Picture.Bitmap.Assign(Steps[_StepPos]);
 DialogSave.Execute;
 if (FileExists(DialogSave.FileName)) then if (MessageDlg('Die Datei existiert bereits. Überschreiben?',MtCustom, [MbYes, MbNo], 0) = mrNo) then exit;
 Picture.SaveToFile(DialogSave.FileName);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
 S : AnsiString;
 i : integer;
begin
 Callback.Code := nil;
 Tool1 := TFarbFilter.Create(self,ToolWahl,STool,nil,@Steps,@TForm1.ReadStepPos,@TForm1.WriteStepPos,@TForm1.GetColor);
 Tool2 := TSchwarzAddition.Create(self,ToolWahl,STool,Tool1.Vorschau,@Steps,@TForm1.ReadStepPos,@TForm1.WriteStepPos);
 Tool3 := TPunktentferner.Create(self,ToolWahl,STool,Tool2.Vorschau,@Steps,@TForm1.ReadStepPos,@TForm1.WriteStepPos,@TForm1.GetColor);
 Pos.Top := 0;
 Pos.Left := 0;
 Pos.Bottom := Bild.Height;
 Pos.Right := Bild.Width;
 Picture := TPicture.Create;
 S := '';
 i := 1;
 while (ParamStr(i) <> '') do begin S := S + ParamStr(i); inc(i); end;
 if (FileExists(S)) then OpenPic(S);
 Bild.OnPaint := @DoPaint;
 DoPaint(Sender);
end;

procedure TForm1.DoForward(Sender: TObject);
begin
 if (_StepPos < Length(Steps) - 1) then inc(_StepPos);
 WriteStepPos(_StepPos);
end;

procedure TForm1.ImgResize(Sender: TObject);
begin
 if (PairSplitterSide.Checked) and (_StepPos >= 0) and (_StepPos < Length(Steps)) and (Steps[_StepPos].Height > 0) then begin
  if (Steps[_StepPos].Width / Steps[_StepPos].Height > PairSplitterSide2.Width / PairSplitterSide2.Height) then begin
   Bild.Left := 0;
   Bild.Width := PairSplitterSide2.Width;
   Bild.Height := Round(Bild.Width / Steps[_StepPos].Width * Steps[_StepPos].Height);
   Bild.Top := (PairSplitterSide2.Height - Bild.Height) div 2;
  end else begin
   Bild.Top := 0;
   Bild.Height := PairSplitterSide2.Height;
   Bild.Width := Round(Bild.Height / Steps[_StepPos].Height * Steps[_StepPos].Width);
   Bild.Left := (PairSplitterSide2.Width - Bild.Width) div 2;
  end;
 end else begin
  Bild.Top := 0;
  Bild.Left := 0;
  Bild.Width := PairSplitterSide2.Width;
  Bild.Height := PairSplitterSide2.Height;
 end;
end;

procedure TForm1.ButtonOpenClick(Sender: TObject);
begin
 DialogOpen.Execute;
 OpenPic(DialogOpen.FileName);
end;

procedure TForm1.OpenPic(const FileName : AnsiString);
var i : integer;
begin
 if (not FileExists(FileName)) then exit;
 Picture.LoadFromFile(FileName);
 for i := 1 to Length(Steps) - 1 do Steps[i].Free;
 i := 0;
 if (Length(Steps) = 0) then i := 1;
 SetLength(Steps,1);
 if (i = 1) then Steps[0] := TBitmap.Create;
 Steps[0].Assign(Picture.Bitmap);
 WriteStepPos(0);
 ActualizeTools;
 DoPaint(nil);
end;

procedure TForm1.DoPaint(Sender : TObject);
var
 Takt     : integer;
 TextRect : TRect;
begin
 if (Operating) then exit;
 if (Pos.Left > 0) then Pos.Left := 0;
 if (Pos.Top > 0) then Pos.Top := 0;
 Pos.Right := Round(Pos.Left + Bild.Width * Zoom);
 Pos.Bottom := Round(Pos.Top + Bild.Height * Zoom);
 if (Pos.Right < Bild.Width) then begin
  Pos.Right := Bild.Width;
  Pos.Left := Round(Pos.Right - Bild.Width * Zoom);
 end;
 if (Pos.Bottom < Bild.Height) then begin
  Pos.Bottom := Bild.Height;
  Pos.Top := Round(Pos.Bottom - Bild.Height * Zoom);
 end;
 Bild.Canvas.Rectangle(-1,-1,Bild.Width,Bild.Height);
 Bild.Canvas.Font.Size := 30;
 Takt := 0;
 TextRect.Top := 0;
 while (TextRect.Top < Bild.Height) do begin
  if (Takt = 0) then Takt := 200
  else Takt := 0;
  TextRect.Left := - Takt;
  while (TextRect.Left < Bild.Width) do begin
   TextRect.Right := TextRect.Left + 300;
   TextRect.Bottom := TextRect.Top + 50;
   Bild.Canvas.TextRect(TextRect,TextRect.Left,TextRect.Top,'TRANSPARENT');
   TextRect.Left := TextRect.Left + 400;
  end;
  TextRect.Top := TextRect.Top + 70;
 end;
 if (_StepPos >= 0) and (_StepPos < Length(Steps)) then Bild.Canvas.StretchDraw(Pos,Steps[_StepPos]);
 Bild.Update;
end;

procedure TForm1.SetZoom(Sender: TObject);
begin
 if (Sender = TrackBar1) then NewZoom := TrackBar1.Position / TrackBar1.Max * MaxZoom;
 if (NewZoom > MaxZoom) then NewZoom := MaxZoom;
 if (NewZoom < MinZoom) then NewZoom := MinZoom;
 if (Sender <> Bild) then begin
  CurPos.X := Bild.Width div 2;
  CurPos.Y := Bild.Height div 2;
 end;
 Pos.Left := Pos.Left - Round((Bild.Width * NewZoom - Bild.Width * Zoom) * ((CurPos.X - Pos.Left) / (Pos.Right - Pos.Left)));
 Pos.Top := Pos.Top - Round((Bild.Height * NewZoom - Bild.Height * Zoom) * ((CurPos.Y - Pos.Top) / (Pos.Bottom - Pos.Top)));
 Zoom := NewZoom;
 DoPaint(Sender);
end;

procedure TForm1.BildMausMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
 CurPos.X := X;
 CurPos.Y := Y;
 if (Moveing) then begin
  Pos.Left := StartPicPos.X + X - StartCurPos.X;
  Pos.Top := StartPicPos.Y + Y - StartCurPos.Y;
  DoPaint(Sender);
 end;
end;

procedure TForm1.BildMausUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 Moveing := false;
end;

procedure TForm1.BildMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
 NewZoom := Zoom / Speed;
 SetZoom(Sender);
end;

procedure TForm1.BildMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
 NewZoom := Zoom * Speed;
 SetZoom(Sender);
end;

procedure TForm1.GetColor(_Callback : TMethod);
begin
 Callback := _Callback;
 Screen.Cursor := crCross;
end;

procedure TForm1.BildMausDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var R,G,B : Word;
begin
 StartPicPos.X := Pos.Left;
 StartPicPos.Y := Pos.Top;
 StartCurPos.X := X;
 StartCurPos.Y := Y;
 Moveing := true;
 if (Callback.Code <> nil) then begin
  Screen.Cursor := crDefault;
  R := RGBAColor(Bild.Canvas.Pixels[X-1,Y+1]).R;
  G := RGBAColor(Bild.Canvas.Pixels[X-1,Y+1]).G;
  B := RGBAColor(Bild.Canvas.Pixels[X-1,Y+1]).B;
  R := R + RGBAColor(Bild.Canvas.Pixels[X,Y+1]).R;
  G := G + RGBAColor(Bild.Canvas.Pixels[X,Y+1]).G;
  B := B + RGBAColor(Bild.Canvas.Pixels[X,Y+1]).B;
  R := R + RGBAColor(Bild.Canvas.Pixels[X+1,Y+1]).R;
  G := G + RGBAColor(Bild.Canvas.Pixels[X+1,Y+1]).G;
  B := B + RGBAColor(Bild.Canvas.Pixels[X+1,Y+1]).B;
  R := R + RGBAColor(Bild.Canvas.Pixels[X-1,Y]).R;
  G := G + RGBAColor(Bild.Canvas.Pixels[X-1,Y]).G;
  B := B + RGBAColor(Bild.Canvas.Pixels[X-1,Y]).B;
  R := R + RGBAColor(Bild.Canvas.Pixels[X,Y]).R;
  G := G + RGBAColor(Bild.Canvas.Pixels[X,Y]).G;
  B := B + RGBAColor(Bild.Canvas.Pixels[X,Y]).B;
  R := R + RGBAColor(Bild.Canvas.Pixels[X+1,Y]).R;
  G := G + RGBAColor(Bild.Canvas.Pixels[X+1,Y]).G;
  B := B + RGBAColor(Bild.Canvas.Pixels[X+1,Y]).B;
  R := R + RGBAColor(Bild.Canvas.Pixels[X-1,Y-1]).R;
  G := G + RGBAColor(Bild.Canvas.Pixels[X-1,Y-1]).G;
  B := B + RGBAColor(Bild.Canvas.Pixels[X-1,Y-1]).B;
  R := R + RGBAColor(Bild.Canvas.Pixels[X,Y-1]).R;
  G := G + RGBAColor(Bild.Canvas.Pixels[X,Y-1]).G;
  B := B + RGBAColor(Bild.Canvas.Pixels[X,Y-1]).B;
  R := R + RGBAColor(Bild.Canvas.Pixels[X+1,Y-1]).R;
  G := G + RGBAColor(Bild.Canvas.Pixels[X+1,Y-1]).G;
  B := B + RGBAColor(Bild.Canvas.Pixels[X+1,Y-1]).B;
  FilterColor.R := R div 9;
  FilterColor.G := G div 9;
  FilterColor.B := B div 9;
  FilterColor.A := 255;
  TCallback(Callback)(FilterColor);
  Callback.Code := nil;
 end;
end;

procedure TForm1.Back(Sender: TObject);
begin
 if (_StepPos > 0) then dec(_StepPos);
 WriteStepPos(_StepPos);
end;

procedure TForm1.WriteStepPos(Pos : integer);
begin
 _StepPos := Pos;
 ImgResize(nil);
 DoPaint(nil);
 if (Pos >= Length(Steps) - 1) then ButtonForward.Enabled := false
 else ButtonForward.Enabled := true;
 if (Pos <= 0) then ButtonBack.Enabled := false
 else ButtonBack.Enabled := true;
end;

function TForm1.ReadStepPos : integer;
begin
 Result := _StepPos;
end;

initialization
  {$I Unit1.lrs}

end.

